from server import *
import json
from jobs import date_json_handler

@app.route('/stats')
@register_navbar('Statistiken', icon='stats')
@mod_required
def stats():
	return render_template('stats.html')

statsqueries = {}
statsqueries['formats_views'] = "SELECT formats.description AS x, count(DISTINCT log.id) AS y FROM log JOIN videos ON (videos.id = log.obj_id) JOIN formats ON (formats.id = videos.video_format) GROUP BY formats.id"
statsqueries['course_count'] = "SELECT semester AS x, count(id) AS y FROM courses GROUP BY semester"
statsqueries['lectures_count'] = "SELECT semester AS x, count(lectures.id) AS y FROM lectures JOIN courses ON (courses.id = lectures.course_id) GROUP BY semester"
statsqueries['categories_courses'] = "SELECT courses.subject AS x, count(courses.id) AS y FROM courses GROUP BY courses.subject ORDER BY y DESC LIMIT 100"
statsqueries['organizer_courses'] = "SELECT courses.organizer AS x, count(courses.id) AS y FROM courses GROUP BY courses.organizer ORDER BY y DESC LIMIT 100"
statsqueries['categories_lectures'] = "SELECT courses.subject AS x, count(lectures.id) AS y FROM lectures JOIN courses ON (courses.id = lectures.course_id) WHERE lectures.visible GROUP BY courses.subject ORDER BY y DESC LIMIT 100"
statsqueries['lecture_views'] = "SELECT lectures.time AS x, count(DISTINCT log.id) AS y FROM log JOIN videos ON (videos.id = log.obj_id) JOIN lectures ON (lectures.id = videos.lecture_id) WHERE (lectures.course_id = ?) GROUP BY lectures.id ORDER BY lectures.time"

def plotly_date_handler(obj):
	return obj.strftime("%Y-%m-%d %H:%M:%S")

@app.route('/stats/generic/<req>')
@app.route('/stats/generic/<req>/<param>')
@mod_required
def stats_generic(req, param=None):
	if req not in statsqueries:
		return 404, 'Not found'
	rows = query(statsqueries[req], *(statsqueries[req].count('?')*[param]))
	res = {'x':[], 'y':[]}
	for row in rows:
		res['x'].append(row['x'])
		res['y'].append(row['y'])
	return Response(json.dumps(res, default=plotly_date_handler),  mimetype='application/json')

@app.route('/stats/viewsperday')
@mod_required
def stats_viewsperday():
	start = None
	end = None
	courseid = request.args.get('course', None)
	lectureid = request.args.get('lecture', None)
	videoid = request.args.get('video', None)
	rows = query('''SELECT log.id AS id, log.time AS time, formats.description AS fmt
			FROM log
			JOIN videos ON (videos.id = log.obj_id)
			JOIN lectures ON (lectures.id = videos.lecture_id)
			JOIN courses ON (courses.id = lectures.course_id)
			JOIN formats ON (formats.id = videos.video_format)
			WHERE (? OR courses.id = ?) AND (? OR lectures.id = ?) AND (? OR videos.id = ?)''',
			courseid == None, courseid, lectureid == None, lectureid, videoid == None, videoid)
	data = {}
	if not end:
		end = date.today()
	first = None
	fmts = ['total']
	for e in rows:
		key = e['time'].date()
		if not first or key < first:
			first = key
		if key not in data:
			data[key] = {'total': set()}
		if e['fmt'] not in data[key]:
			data[key][e['fmt']] = set()
		data[key]['total'].add(e['id'])
		data[key][e['fmt']].add(e['id'])
		if e['fmt'] not in fmts:
			fmts.append(e['fmt'])
	if not start:
		start = first
	res = {'times': [], 'views': [{'name': name, 'vals': []} for name in fmts]}
	while start and start <= end:
		res['times'].append(start.isoformat())
		for d in res['views']:
			d['vals'].append(len(data.get(start, {}).get(d['name'], set())))
		start += timedelta(days=1)
	return Response(json.dumps(res, default=date_json_handler),  mimetype='application/json')

